/***************************************************************************
                                Dummy.h
                          -------------------
    begin                : Sat Apr  5 22:07:20 CST 2003
    copyright            : (C) 2005 by Andrew Ruder
    email                : aeruder@ksu.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

@class TalkSoupDummyProtocolClass;

#ifndef DUMMY_H
#define DUMMY_H

#import <Foundation/NSObject.h>

@interface TalkSoupDummyProtocolClass : NSObject
@end

#endif
